use mongod::{
    assert_reference_of,
    derive::{Model, Referencable},
    Model, Referencable, Reference, Validate,
};
use serde::{Deserialize, Serialize};

use crate::library::artist::Artist;

use super::metadata::AudioMetadata;

#[derive(Debug, Clone, Serialize, Deserialize, Model, Referencable)]
pub struct Track {
    pub _id: String,
    pub path: String,
    pub title: String,
    pub album_id: Option<Reference>,
    pub artist_id: Option<Reference>,
    pub meta: Option<AudioMetadata>,
}

impl Track {
    pub async fn create(data: &serde_json::Map<String, serde_json::Value>) {
        let mut t = Self {
            _id: uuid::Uuid::new_v4().to_string(),
            path: data.get("path").unwrap().as_str().unwrap().to_string(),
            title: data.get("title").unwrap().as_str().unwrap().to_string(),
            album_id: None,
            artist_id: None,
            meta: data.get("meta").map(|x| AudioMetadata(x.clone())),
        };
        t.insert().await.unwrap();
        t.update(&serde_json::to_value(&data).unwrap())
            .await
            .unwrap();
    }
}

impl Validate for Track {
    async fn validate(&self) -> Result<(), String> {
        if let Some(artist_id) = &self.artist_id {
            assert_reference_of!(artist_id, Artist);
        }

        if let Some(album_id) = &self.artist_id {
            assert_reference_of!(album_id, Artist);
        }

        Ok(())
    }
}
